package it.unibo.sdls.sampleproject.dao.ejb3;

import java.util.List;

import javax.ejb.Stateless;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Transient;

import it.unibo.sdls.sampleproject.dao.Author;
import it.unibo.sdls.sampleproject.dao.Book;
import it.unibo.sdls.sampleproject.dao.BookDAO;
import it.unibo.sdls.sampleproject.dao.Publisher;

@Stateless @TransactionManagement(TransactionManagementType.CONTAINER)
public class BookDAOBean implements BookDAO {


	@PersistenceContext(unitName = "SampleProjectUnit") EntityManager em;
	
	@SuppressWarnings("unchecked")
     private List<Book> findBy(String field,String val)
	{
		return em.createQuery("SELECT a FROM Book a WHERE :f LIKE :v ")
		.setParameter("v", val)
		.setParameter("f",field)
		.getResultList();
	}
     
	public int addBook(Book book) {
		em.persist(book);
		return 0;
	}

	public int deleteBook(int id) {
		em.remove(em.find(Book.class, id));
		return 0;
	}

	public Book getBookById(int id) {
		return em.find(Book.class, id);
	}

	public Book getBookByISBN10(String isbn10) {
		return findBy("isbn10",isbn10).get(0);
	}

	public Book getBookByISBN13(String isbn13) {
		return findBy("isbn13",isbn13).get(0);
	}

	public Book getBookByTitle(String title) {
		return findBy("title",title).get(0);
	}
	
	@SuppressWarnings("unchecked")
	public List<Book> getAllBooks() {
		return em.createQuery("SELECT a FROM Book a" ).getResultList();
	}

	@SuppressWarnings("unchecked")
	public List<Book> getAllBooksByAuthor(Author author) {
		int id=author.getId();
		return em.createQuery("SELECT a FROM Written_by a WHERE Author_ID LIKE :id")
		.setParameter("id", id)
		.getResultList();
	}

	@SuppressWarnings("unchecked")
	public List<Book> getAllBooksByPublisher(Publisher publisher) {
		int id=publisher.getId();
		return em.createQuery("SELECT a FROM Book a WHERE Publisher LIKE :id")
		.setParameter("id", id)
		.getResultList();
	}

}
